# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022-2026 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""This module contains the code to generate dependency trees from registries."""

from pathlib import Path
from typing import Dict, List, Optional, Set, Tuple

import yaml

from aea.configurations.constants import (
    AGENT,
    CONNECTION,
    CONTRACT,
    CUSTOM,
    DEFAULT_AEA_CONFIG_FILE,
    DEFAULT_CONNECTION_CONFIG_FILE,
    DEFAULT_CONTRACT_CONFIG_FILE,
    DEFAULT_CUSTOM_COMPONENT_CONFIG_FILE,
    DEFAULT_PROTOCOL_CONFIG_FILE,
    DEFAULT_SERVICE_CONFIG_FILE,
    DEFAULT_SKILL_CONFIG_FILE,
    PACKAGE_TYPE_TO_CONFIG_FILE,
    PROTOCOL,
    SERVICE,
    SKILL,
    VENDOR,
)
from aea.configurations.data_types import PackageId, PublicId
from aea.exceptions import AEAPackageLoadingError
from aea.helpers.yaml_utils import _AEAYamlDumper, _AEAYamlLoader

COMPONENTS = [
    (AGENT, DEFAULT_AEA_CONFIG_FILE),
    (SKILL, DEFAULT_SKILL_CONFIG_FILE),
    (CONTRACT, DEFAULT_CONTRACT_CONFIG_FILE),
    (CONNECTION, DEFAULT_CONNECTION_CONFIG_FILE),
    (PROTOCOL, DEFAULT_PROTOCOL_CONFIG_FILE),
    (CUSTOM, DEFAULT_CUSTOM_COMPONENT_CONFIG_FILE),
    (SERVICE, DEFAULT_SERVICE_CONFIG_FILE),
]


def load_yaml(file_path: Path) -> Tuple[Dict, List[Dict]]:
    """Load yaml file."""
    with open(file_path, "r", encoding="utf-8") as fp:
        item_config, *extra_data = list(yaml.load_all(fp, Loader=_AEAYamlLoader))
        return item_config, extra_data


def dump_yaml(
    file_path: Path, data: Dict, extra_data: Optional[List[Dict]] = None
) -> None:
    """Dump yaml file."""
    if extra_data is not None and len(extra_data) > 0:
        with open(file_path, "w+", encoding="utf-8") as fp:
            yaml.dump_all([data, *extra_data], fp, Dumper=_AEAYamlDumper)
    else:
        with open(file_path, "w+", encoding="utf-8") as fp:
            yaml.dump(data, fp, Dumper=_AEAYamlDumper)


def to_plural(string: str) -> str:
    """Convert component to plural"""
    return string + "s"


def reduce_sets(list_of_sets: List[Set]) -> Set[PackageId]:
    """Reduce a list of sets to one dimentional set."""
    reduced: Set[PackageId] = set()
    for s in list_of_sets:
        reduced = reduced.union(s)
    return reduced


def to_package_id(public_id: str, package_type: str) -> PackageId:
    """Convert to public id."""
    return PackageId(package_type, PublicId.from_str(public_id)).without_hash()


class DependencyTree:
    """This class represents the dependency tree for a registry."""

    @staticmethod
    def get_all_dependencies(item_config: Dict) -> List[PackageId]:
        """Returns a list of all available dependencies."""
        return list(
            reduce_sets(
                [
                    {
                        to_package_id(dependency, component_type)
                        for dependency in item_config.get(
                            to_plural(component_type), set()
                        )
                    }
                    for component_type, _ in COMPONENTS[1:]
                ]
            )
        )

    @classmethod
    def resolve_tree(cls, dependency_list: Dict[PackageId, List[PackageId]]) -> Dict:
        """
        Resolve dependency tree.

        :param dependency_list: the adjacency list of the dependency graph
        :return: the dependency tree
        """
        tree: Dict[PackageId, Dict] = {root_node: {} for root_node in dependency_list}
        # auxiliary variables to keep track during recursive calls :
        #  visited: a set for fast checking of already visited nodes in the current path of DFS
        #  stack:   the current stack of nodes in the depth-first visiting of nodes
        visited: Set[PackageId] = set()
        stack: List[PackageId] = []
        cls._resolve_tree_aux(dependency_list, tree, visited, stack)

        # _resolve_tree_aux did side-effect on the 'tree' object
        return tree

    @classmethod
    def _resolve_tree_aux(
        cls,
        dependency_list: Dict[PackageId, List[PackageId]],
        tree: Dict,
        visited: Set[PackageId],
        stack: List[PackageId],
    ) -> None:
        """
        Resolve dependency tree (auxiliary method of resolve_tree).

        IMPORTANT: This method does side-effect on the parameter 'tree', 'visited' and 'stack'.

        :param dependency_list: the adjacency list of hte dependency graph
        :param tree: the dependency tree
        :param visited: the set of visited nodes in the current path
        :param stack: the current path stored in a LIFO data structure
        """
        for root_package in tree:
            if root_package in visited:
                # cycle found; raise error
                cls._raise_circular_dependency_error(root_package, stack)

            # add current package to the stack - needed to check if there are circular dependencies
            visited.add(root_package)
            stack.append(root_package)

            # build the root of the subtree according to the dependency list
            root_dependencies: List[PackageId] = dependency_list.get(root_package, [])
            tree[root_package] = {p: {} for p in root_dependencies}
            # do a recursive call on the newly created subtree
            cls._resolve_tree_aux(dependency_list, tree[root_package], visited, stack)

            # the current search node can be removed from the stack
            visited.discard(root_package)
            stack.pop()

    @classmethod
    def flatten_tree(
        cls, dependency_tree: Dict, flat_tree: List[List[PackageId]], level: int
    ) -> None:
        """Flatten tree."""
        try:
            flat_tree[level]
        except IndexError:
            flat_tree.insert(level, [])
        for package, dependencies in dependency_tree.items():
            flat_tree[level].append(package)
            cls.flatten_tree(dependencies, flat_tree, level + 1)

    @staticmethod
    def find_packages_in_a_project(
        project_dir: Path,
    ) -> List[Tuple[str, Path]]:
        """Find packages in an AEA project."""

        packages = []
        for package_type, config_file in COMPONENTS:
            _packages = [
                *Path(project_dir, VENDOR).glob(f"*/{package_type}s/*/{config_file}"),
                *Path(project_dir, f"{package_type}s").glob(f"*/{config_file}"),
            ]
            packages += [
                (package_type, package_path.parent) for package_path in _packages
            ]

        return packages

    @staticmethod
    def find_packages_in_a_local_repository(
        packages_dir: Path,
    ) -> List[Tuple[str, Path]]:
        """Find packages in a local repository."""

        packages = []
        for package_type, config_file in COMPONENTS:
            packages += [
                (package_type, package_path.parent)
                for package_path in Path(packages_dir).glob(
                    f"*/{package_type}s/*/{config_file}"
                )
            ]

        return packages

    @classmethod
    def generate(
        cls, packages_dir: Path, from_project: bool = False
    ) -> List[List[PackageId]]:
        """Returns PublicId to hash mapping."""
        package_to_dependency_mappings = {}
        packages_list = []

        if from_project or (packages_dir / VENDOR).exists():
            packages_list = cls.find_packages_in_a_project(packages_dir)
        else:
            packages_list = cls.find_packages_in_a_local_repository(packages_dir)

        for package_type, package_path in packages_list:
            item_config, _ = load_yaml(
                package_path / PACKAGE_TYPE_TO_CONFIG_FILE[package_type]
            )
            item_config["name"] = item_config.get("name", item_config.get("agent_name"))
            public_id = PublicId.from_json(item_config)
            package_to_dependency_mappings[
                to_package_id(str(public_id), package_type)
            ] = cls.get_all_dependencies(item_config)

        flat_tree: List[List[PackageId]] = []
        flat_tree_dirty: List[List[PackageId]] = []
        dirty_packages: List[PackageId] = []

        dep_tree = cls.resolve_tree(package_to_dependency_mappings)
        cls.flatten_tree(dep_tree, flat_tree_dirty, 0)

        for tree_level in reversed(flat_tree_dirty[:-1]):
            flat_tree.append([])
            for package in tree_level:
                if package not in dirty_packages and package not in flat_tree[-1]:
                    flat_tree[-1].append(package)
            dirty_packages += tree_level

        return flat_tree

    @classmethod
    def _raise_circular_dependency_error(
        cls, package: PackageId, stack: List[PackageId]
    ) -> None:
        # find first occurrence of the package in the stack:
        start = stack.index(package)
        start_node = stack[start]
        cycle = stack[start:]
        if len(cycle) == 1:
            raise AEAPackageLoadingError(
                f"Found a self-loop dependency while resolving dependency tree in package {cycle[0]}"
            )
        raise AEAPackageLoadingError(
            "Found a circular dependency while resolving dependency tree: "
            + " -> ".join(map(str, cycle + [start_node]))
        )



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022-2026 Valory AG
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""This module contains the behaviours for the 'test task skill' skill."""

from typing import Optional

from aea.skills.behaviours import TickerBehaviour
from aea.skills.tasks import Task

from packages.fetchai.skills.task_test_skill.tasks import SimpleTask


class TaskBehaviour(TickerBehaviour):
    """Echo behaviour."""

    task_id: Optional[int]
    task: Task

    def setup(self) -> None:
        """Set up the behaviour."""
        self.context.logger.info("Task Behaviour: setup method called.")
        self.task = SimpleTask("some data")
        self.task_id = None

    def act(self) -> None:
        """Act according to the behaviour."""
        self.context.logger.info("Task Behaviour: act method called.")
        if self.task_id is None:
            self.set_task()
        else:
            self.get_task_result()

    def set_task(self) -> None:
        """Set background task to run."""
        if self.task_id:
            return
        self.task_id = self.context.task_manager.enqueue_task(self.task)
        self.context.logger.info("Task set.")

    def get_task_result(self) -> None:
        """Get result of the task."""
        if self.task_id is None:
            return
        async_result = self.context.task_manager.get_task_result(self.task_id)
        if not async_result.ready():
            return
        self.context.logger.info(f"Task result is ready: {async_result.get()}")
        self.task_id = None

    def teardown(self) -> None:
        """Teardown the behaviour."""
        self.context.logger.info("Task Behaviour: teardown method called.")



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2024-2026 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""RPC rotation support for EthereumApi as a web3 middleware.

When multiple RPC endpoints are provided (comma-separated), the
:class:`RPCRotationMiddleware` automatically fails over to healthy
endpoints on rate-limit, connection, or quota errors.  With a single
RPC URL the middleware is a transparent pass-through.
"""

import logging
import threading
import time
from typing import Any, Dict, FrozenSet, List, Optional, Union

from aea_ledger_ethereum.chainlist import enrich_rpc_urls
from web3 import AsyncWeb3, HTTPProvider, Web3
from web3.middleware.base import Web3MiddlewareBuilder
from web3.types import RPCEndpoint, RPCResponse

_logger = logging.getLogger("aea.ledger_apis.ethereum.rpc_rotation")

MakeRequestFn = Any  # web3 typing alias

# ---------------------------------------------------------------------------
# Write RPC methods — retried only on clear pre-send failures
# ---------------------------------------------------------------------------

WRITE_RPC_METHODS: FrozenSet[str] = frozenset(
    {"eth_sendRawTransaction", "eth_sendTransaction"}
)

# ---------------------------------------------------------------------------
# Error classification signals
# ---------------------------------------------------------------------------

RATE_LIMIT_SIGNALS = ("429", "rate limit", "too many requests", "ratelimit")

CONNECTION_SIGNALS = (
    "timeout",
    "timed out",
    "connection refused",
    "connection reset",
    "connection error",
    "connection aborted",
    "name resolution",
    "dns",
    "no route to host",
    "network unreachable",
    "max retries exceeded",
    "read timeout",
    "connect timeout",
    "remote end closed",
    "broken pipe",
    "404",
    "not found",
)

QUOTA_SIGNALS = (
    "exceeded the quota",
    "exceeded quota",
    "quota usage",
    "quota exceeded",
    "allowance exceeded",
)

SERVER_ERROR_SIGNALS = (
    "500",
    "502",
    "503",
    "504",
    "internal server error",
    "bad gateway",
    "service unavailable",
    "gateway timeout",
)

FD_EXHAUSTION_SIGNALS = ("too many open files", "oserror(24", "errno 24")

# ---------------------------------------------------------------------------
# Backoff durations (seconds)
# ---------------------------------------------------------------------------

RATE_LIMIT_BACKOFF = 10.0
QUOTA_EXCEEDED_BACKOFF = 300.0
CONNECTION_ERROR_BACKOFF = 30.0
SERVER_ERROR_BACKOFF = 15.0
FD_EXHAUSTION_BACKOFF = 60.0

_BACKOFF_MAP: Dict[str, float] = {
    "rate_limit": RATE_LIMIT_BACKOFF,
    "connection": CONNECTION_ERROR_BACKOFF,
    "quota": QUOTA_EXCEEDED_BACKOFF,
    "server": SERVER_ERROR_BACKOFF,
    "fd_exhaustion": FD_EXHAUSTION_BACKOFF,
}

# ---------------------------------------------------------------------------
# Retry configuration
# ---------------------------------------------------------------------------

MAX_RETRIES = 6
RETRY_DELAY = 1.0  # base delay between retries (exponential backoff)
MAX_RETRY_DELAY = 5.0  # cap on retry delay
ROTATION_COOLDOWN = 2.0  # min time between rotations to prevent cascade


# ---------------------------------------------------------------------------
# Public helpers
# ---------------------------------------------------------------------------


def parse_rpc_urls(address: str) -> List[str]:
    """Parse RPC URL(s) from an *address* string.

    Supports a single URL or a comma-separated list.
    Returns a list with at least one URL.

    :param address: single RPC URL or comma-separated list of URLs.
    :return: list of parsed RPC URL strings.
    """
    if "," in address:
        urls = [url.strip() for url in address.split(",") if url.strip()]
        return urls if urls else [address]
    return [address.strip()]


def classify_error(error: Exception) -> str:
    """Classify an RPC error into a category.

    Returns one of:
    ``"rate_limit"``, ``"connection"``, ``"quota"``, ``"server"``,
    ``"fd_exhaustion"``, or ``"unknown"``.

    :param error: the exception raised by the RPC call.
    :return: error category string.
    """
    err_text = str(error).lower()

    if any(s in err_text for s in FD_EXHAUSTION_SIGNALS):
        return "fd_exhaustion"
    if any(s in err_text for s in RATE_LIMIT_SIGNALS):
        return "rate_limit"
    if any(s in err_text for s in QUOTA_SIGNALS):
        return "quota"
    if any(s in err_text for s in CONNECTION_SIGNALS):
        return "connection"
    if any(s in err_text for s in SERVER_ERROR_SIGNALS):
        return "server"
    return "unknown"


# ---------------------------------------------------------------------------
# Middleware
# ---------------------------------------------------------------------------


class RPCRotationMiddleware(Web3MiddlewareBuilder):
    """Web3 middleware that rotates RPC endpoints on transport failures.

    Manages a pool of :class:`~web3.HTTPProvider` instances with
    per-endpoint health tracking, automatic failover, and
    exponential-backoff retry logic.

    For **write** operations (``eth_sendRawTransaction``,
    ``eth_sendTransaction``) only clear pre-send connection failures are
    retried to prevent double-submission.

    Usage::

        rpc_rotation = RPCRotationMiddleware.build(
            w3,
            rpc_urls=["https://rpc1.example.com", "https://rpc2.example.com"],
            request_kwargs={"timeout": 10},
            chain_id=100,
        )
        web3.middleware_onion.add(rpc_rotation)
    """

    # Set by build()
    _rpc_urls: List[str]
    _request_kwargs: Dict[str, Any]
    _providers: List[HTTPProvider]
    _current_index: int
    _backoff_until: Dict[int, float]
    _lock: threading.Lock
    _rotation_enabled: bool
    _last_rotation_time: float

    @classmethod
    def build(  # pylint: disable=arguments-differ
        cls,
        w3: Union[AsyncWeb3, Web3],
        rpc_urls: List[str],
        request_kwargs: Optional[Dict[str, Any]] = None,
        chain_id: Optional[int] = None,
    ) -> "RPCRotationMiddleware":
        """Build the middleware.

        :param w3: the Web3 instance.
        :param rpc_urls: list of RPC endpoint URL strings (required).
        :param request_kwargs: dict forwarded to each HTTPProvider.
        :param chain_id: optional chain ID for Chainlist fallback enrichment.
        :return: configured :class:`RPCRotationMiddleware` instance.
        """
        if request_kwargs is None:
            request_kwargs = {}

        rpc_urls = enrich_rpc_urls(rpc_urls, chain_id=chain_id)

        mw = cls(w3)
        mw._rpc_urls = rpc_urls  # pylint: disable=protected-access
        mw._request_kwargs = request_kwargs  # pylint: disable=protected-access
        mw._providers = [  # pylint: disable=protected-access
            HTTPProvider(endpoint_uri=url, request_kwargs=request_kwargs)
            for url in rpc_urls
        ]
        mw._current_index = 0  # pylint: disable=protected-access
        mw._backoff_until = {}  # pylint: disable=protected-access
        mw._lock = threading.Lock()  # pylint: disable=protected-access
        mw._rotation_enabled = len(rpc_urls) > 1  # pylint: disable=protected-access
        mw._last_rotation_time = 0.0  # pylint: disable=protected-access
        return mw

    def __call__(self, w3: Any = None) -> "RPCRotationMiddleware":
        """Allow this pre-built instance to be stored directly in the middleware onion.

        web3's ``combine_middleware`` calls ``mw(w3)`` on each entry; returning
        ``self`` ensures the already-initialised instance is reused unchanged.

        :param w3: web3 instance (ignored — already set on build).
        :return: this middleware instance.
        """
        return self

    # ------------------------------------------------------------------
    # Public introspection
    # ------------------------------------------------------------------

    @property
    def current_rpc_url(self) -> str:
        """Return the currently active RPC URL."""
        return self._rpc_urls[self._current_index]

    @property
    def rpc_count(self) -> int:
        """Return the number of configured RPC endpoints."""
        return len(self._rpc_urls)

    # ------------------------------------------------------------------
    # Per-RPC health tracking
    # ------------------------------------------------------------------

    def _mark_rpc_backoff(self, index: int, seconds: float) -> None:
        """Mark an RPC as temporarily unavailable for *seconds*."""
        self._backoff_until[index] = time.monotonic() + seconds

    def _is_rpc_healthy(self, index: int) -> bool:
        """Return ``True`` if the RPC at *index* is not in backoff."""
        return time.monotonic() >= self._backoff_until.get(index, 0.0)

    # ------------------------------------------------------------------
    # Provider rotation
    # ------------------------------------------------------------------

    def _rotate(self) -> bool:
        """Rotate to the next healthy RPC endpoint.

        :return: ``True`` if a rotation occurred, ``False`` otherwise.
        """
        with self._lock:
            n = len(self._rpc_urls)
            if n <= 1:
                return False

            now = time.monotonic()
            if now - self._last_rotation_time < ROTATION_COOLDOWN:
                return False

            best: Optional[int] = None
            for offset in range(1, n):
                candidate = (self._current_index + offset) % n
                if self._is_rpc_healthy(candidate):
                    best = candidate
                    break

            if best is None:
                best = min(
                    (i for i in range(n) if i != self._current_index),
                    key=lambda i: self._backoff_until.get(i, 0.0),
                )

            self._current_index = best
            self._last_rotation_time = now
            _logger.info("Rotated RPC to #%d: %s", best, self._rpc_urls[best])
            return True

    # ------------------------------------------------------------------
    # Error handling
    # ------------------------------------------------------------------

    def _handle_error_and_rotate(self, error: Exception, operation: str) -> bool:
        """Classify *error*, apply backoff, and rotate.

        :param error: the transport-level exception.
        :param operation: human-readable label for log messages.
        :return: ``True`` if the caller should retry.
        """
        category = classify_error(error)

        if category == "fd_exhaustion":
            _logger.error(
                "FD exhaustion detected — pausing ALL RPCs for %ds.",
                int(FD_EXHAUSTION_BACKOFF),
            )
            for i in range(len(self._rpc_urls)):
                self._mark_rpc_backoff(i, FD_EXHAUSTION_BACKOFF)
            self._rotate()
            return True

        if category == "unknown":
            return False

        backoff = _BACKOFF_MAP.get(category, 0.0)
        self._mark_rpc_backoff(self._current_index, backoff)
        _logger.warning(
            "RPC #%d %s error (backoff %ds) during %s: %.120s",
            self._current_index,
            category.upper(),
            int(backoff),
            operation,
            str(error),
        )
        self._rotate()
        return True

    # ------------------------------------------------------------------
    # wrap_make_request
    # ------------------------------------------------------------------

    def wrap_make_request(self, make_request: MakeRequestFn) -> MakeRequestFn:
        """Wrap the JSON-RPC make_request with retry and rotation logic.

        :param make_request: the next function in the middleware chain.
        :return: wrapped make_request function.
        """

        def middleware(method: RPCEndpoint, params: Any) -> RPCResponse:
            """Middleware function to override the request with."""
            if not self._rotation_enabled:
                return make_request(method, params)

            is_write = method in WRITE_RPC_METHODS
            max_retries = min(MAX_RETRIES, len(self._rpc_urls) * 2)
            last_error: Optional[Exception] = None

            for attempt in range(max_retries + 1):
                try:
                    return self._providers[self._current_index].make_request(
                        method, params
                    )
                except Exception as exc:  # pylint: disable=broad-exception-caught
                    last_error = exc
                    category = classify_error(exc)

                    # Write safety: only retry on clear pre-send failures
                    if is_write and category not in ("connection", "fd_exhaustion"):
                        raise

                    should_retry = self._handle_error_and_rotate(exc, str(method))
                    if not should_retry or attempt >= max_retries:
                        raise

                    delay = min(RETRY_DELAY * (2**attempt), MAX_RETRY_DELAY)
                    _logger.info(
                        "%s attempt %d failed, retrying in %.1fs …",
                        method,
                        attempt + 1,
                        delay,
                    )
                    time.sleep(delay)

            raise last_error  # type: ignore[misc]

        return middleware



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022-2026 Valory AG
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""This module contains the tests of the behaviour classes of the erc1155_client skill."""

# pylint: skip-file

from packages.fetchai.protocols.oef_search.message import OefSearchMessage
from packages.fetchai.skills.erc1155_client.behaviours import LEDGER_API_ADDRESS
from packages.fetchai.skills.erc1155_client.tests.intermediate_class import (
    ERC1155ClientTestCase,
)
from packages.valory.protocols.ledger_api.message import LedgerApiMessage


class TestSearchBehaviour(ERC1155ClientTestCase):
    """Test search behaviour of erc1155_client."""

    def test_setup(self):
        """Test the setup method of the search behaviour."""
        # operation
        self.search_behaviour.setup()

        # after
        self.assert_quantity_in_outbox(1)
        has_attributes, error_str = self.message_has_attributes(
            actual_message=self.get_message_from_outbox(),
            message_type=LedgerApiMessage,
            performative=LedgerApiMessage.Performative.GET_BALANCE,
            to=LEDGER_API_ADDRESS,
            sender=str(self.skill.public_id),
            ledger_id=self.strategy.ledger_id,
            address=self.skill.skill_context.agent_address,
        )
        assert has_attributes, error_str

    def test_act_is_searching(self):
        """Test the act method of the search behaviour where is_searching is True."""
        # setup
        self.strategy.is_searching = True
        assert self.strategy.is_searching

        # operation
        self.search_behaviour.act()

        # after
        self.assert_quantity_in_outbox(1)
        has_attributes, error_str = self.message_has_attributes(
            actual_message=self.get_message_from_outbox(),
            message_type=OefSearchMessage,
            performative=OefSearchMessage.Performative.SEARCH_SERVICES,
            to=self.skill.skill_context.search_service_address,
            sender=str(self.skill.public_id),
            query=self.skill.skill_context.strategy.get_location_and_service_query(),
        )
        assert has_attributes, error_str

    def test_act_not_is_searching(self):
        """Test the act method of the search behaviour where is_searching is False."""
        # setup
        self.strategy.is_searching = False

        # operation
        self.search_behaviour.act()

        # after
        self.assert_quantity_in_outbox(0)

    def test_teardown(self):
        """Test the teardown method of the search behaviour."""
        assert self.search_behaviour.teardown() is None
        self.assert_quantity_in_outbox(0)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022-2026 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Test package manager base."""

import json
import logging
import os
import shutil
import tempfile
from collections import OrderedDict
from pathlib import Path
from unittest import mock

import pytest

from aea.cli.packages import package_type_selector_prompt
from aea.configurations.constants import PACKAGES
from aea.configurations.data_types import PackageId, PackageType, PublicId
from aea.helpers.ipfs.base import IPFSHashOnly
from aea.package_manager.v0 import PackageManagerV0
from aea.protocols.generator.common import INIT_FILE_NAME
from aea.test_tools.test_cases import BaseAEATestCase

from tests.conftest import ROOT_DIR
from tests.test_package_manager.test_base import (
    DUMMY_PACKAGE_HASH,
    DUMMY_PACKAGE_ID,
    EXAMPLE_PACKAGE_HASH,
    EXAMPLE_PACKAGE_ID,
    PACKAGE_JSON_FILE,
)

TEST_SKILL_ID = PackageId(
    package_type=PackageType.SKILL,
    public_id=PublicId.from_str(
        "valory/abstract_round_abci:0.1.0:bafybeifh4qtjurq5637ykxexzexca5l4n6t4ujw26tpnern2swajanvhny"
    ),
)


class TestPackageManagerV0(BaseAEATestCase):
    """Test aea package manager."""

    use_packages_dir: bool = True
    packages_json_file: Path

    def setup_method(
        self,
    ) -> None:
        """Setup test."""

        # the current `packages.json` in `packages/` directory follows the v1
        # format so we will have to load it and convert it to v0 format
        packages_v1 = json.loads(PACKAGE_JSON_FILE.read_text(encoding="utf-8"))

        self.packages_json_file = self.packages_dir_path / "packages.json"
        self.packages_json_file.write_text(
            json.dumps({**packages_v1["dev"], **packages_v1["third_party"]})
        )

    def test_initialization(
        self,
    ) -> None:
        """Test object initialization."""

        pm = PackageManagerV0.from_dir(self.packages_dir_path)

        assert len(pm.packages) == len(json.loads(self.packages_json_file.read_text()))
        assert pm.path == self.packages_dir_path
        assert pm.get_package_hash(package_id=EXAMPLE_PACKAGE_ID) is not None
        assert pm.get_package_hash(package_id=DUMMY_PACKAGE_ID) is None

    def test_sync(
        self,
    ) -> None:
        """Test sync."""

        pm = PackageManagerV0(
            path=self.packages_dir_path,
            packages=OrderedDict({DUMMY_PACKAGE_ID: DUMMY_PACKAGE_HASH}),
        )

        with mock.patch.object(pm, "add_package") as update_patch:
            pm.sync()
            update_patch.assert_called_with(
                package_id=DUMMY_PACKAGE_ID.with_hash(
                    "bafybei0000000000000000000000000000000000000000000000000000"
                ),
                with_dependencies=True,
            )

        with pytest.raises(
            ValueError,
            match="Both `update_packages` and `update_hashes` cannot be set to `True`.",
        ):
            pm.sync(update_hashes=True, update_packages=True)

        PACKAGES = {EXAMPLE_PACKAGE_ID: EXAMPLE_PACKAGE_HASH}
        with mock.patch.object(
            pm, "_sync", return_value=[False, PACKAGES, []]
        ) as update_patch:
            pm.sync(update_hashes=True)
            assert pm._packages == PACKAGES

        PACKAGES = {EXAMPLE_PACKAGE_ID: EXAMPLE_PACKAGE_HASH}
        with mock.patch.object(
            pm, "_sync", return_value=[False, PACKAGES, PACKAGES]
        ), mock.patch.object(pm, "update_package") as update_package:
            pm.sync(update_packages=True)
            update_package.assert_called_once_with(package_id=EXAMPLE_PACKAGE_ID)

    def test_update_fingerprints(self, caplog) -> None:
        """Test update fingerprints."""

        package_id = PackageId.from_uri_path("protocol/open_aea/signing/1.0.0")
        package_dir_rel = Path(
            PACKAGES,
            package_id.author,
            package_id.package_type.to_plural(),
            package_id.name,
        )
        original_package = ROOT_DIR / package_dir_rel
        package_hash = IPFSHashOnly.get(str(original_package))

        with tempfile.TemporaryDirectory() as temp_dir:
            packages_dir = Path(temp_dir, PACKAGES)
            temp_package = Path(temp_dir, *package_dir_rel.parts)

            os.makedirs(temp_package.parent)

            shutil.copytree(original_package, temp_package)
            pm = PackageManagerV0(
                path=packages_dir, packages=OrderedDict({package_id: package_hash})
            )

            (temp_package / "__init__.py").write_text("dummy")

            with caplog.at_level(logging.ERROR):
                assert pm.verify() == 1
                assert (
                    "Fingerprints does not match for (protocol, open_aea/signing:1.0.0)"
                    in caplog.text
                )

            pm.update_package_hashes(selector_prompt=package_type_selector_prompt)
            assert pm.verify() == 0


class TestHashUpdate(BaseAEATestCase):
    """Test hash update."""

    use_packages_dir: bool = True

    def test_update_package_hashes(
        self,
    ) -> None:
        """Test update package hash method."""

        packages_v1 = json.loads(PACKAGE_JSON_FILE.read_text(encoding="utf-8"))
        packages = {**packages_v1["dev"], **packages_v1["third_party"]}

        original_hash = packages[EXAMPLE_PACKAGE_ID.to_uri_path]
        packages[EXAMPLE_PACKAGE_ID.to_uri_path] = DUMMY_PACKAGE_HASH

        packages_json_file = self.packages_dir_path / "packages.json"
        packages_json_file.write_text(json.dumps(obj=packages))
        pm = PackageManagerV0.from_dir(self.packages_dir_path)

        pm.update_package_hashes(selector_prompt=package_type_selector_prompt).dump()

        packages_v1_updated = json.loads(packages_json_file.read_text(encoding="utf-8"))

        assert pm.get_package_hash(EXAMPLE_PACKAGE_ID) == original_hash
        assert packages_v1_updated[EXAMPLE_PACKAGE_ID.to_uri_path] == original_hash


class TestVerifyFailure(BaseAEATestCase):
    """Test verify method."""

    use_packages_dir: bool = True

    def test_verify_method(self, caplog) -> None:
        """Test update package hash method."""

        packages_v1 = json.loads(PACKAGE_JSON_FILE.read_text(encoding="utf-8"))
        packages = {**packages_v1["dev"], **packages_v1["third_party"]}

        packages_json_file = self.packages_dir_path / "packages.json"
        packages_json_file.write_text(json.dumps(obj=packages))

        pm = PackageManagerV0.from_dir(self.packages_dir_path)
        assert pm.verify() == 0

        # updating the `packages/open_aea/protocols/signing/__init__.py` file
        init_file = (
            pm.package_path_from_package_id(package_id=EXAMPLE_PACKAGE_ID)
            / INIT_FILE_NAME
        )
        init_file.write_text("dummy")

        with caplog.at_level(logging.ERROR), mock.patch(
            "aea.package_manager.v0.check_fingerprint",
            return_value=True,
        ):
            pm = PackageManagerV0.from_dir(self.packages_dir_path)

            assert pm.verify() == 1
            assert f"Hash does not match for {EXAMPLE_PACKAGE_ID}" in caplog.text
            assert (
                f"Dependency check failed\nHash does not match for {EXAMPLE_PACKAGE_ID}"
                in caplog.text
            )

        with mock.patch(
            "traceback.print_exc",
        ) as print_tb, mock.patch(
            "aea.package_manager.v0.check_fingerprint",
            side_effect=ValueError("expeceted_exception"),
        ):
            pm = PackageManagerV0.from_dir(self.packages_dir_path)
            assert pm.verify() == 1
            print_tb.assert_called_once()

    def test_fingerprint_failure(self, caplog) -> None:
        """Test update package hash method."""

        pm = PackageManagerV0(
            path=self.packages_dir_path,
            packages=OrderedDict(
                {
                    EXAMPLE_PACKAGE_ID: EXAMPLE_PACKAGE_HASH,
                }
            ),
        )

        with caplog.at_level(logging.ERROR), mock.patch(
            "aea.package_manager.v0.check_fingerprint",
            return_value=False,
        ), mock.patch.object(
            pm,
            "iter_dependency_tree",
            return_value=[
                EXAMPLE_PACKAGE_ID,
            ],
        ):
            assert pm.verify() == 1
            assert (
                f"Fingerprints does not match for {EXAMPLE_PACKAGE_ID}" in caplog.text
            )

    def test_missing_hash(self, caplog) -> None:
        """Test update package hash method."""

        pm: PackageManagerV0 = PackageManagerV0(
            path=self.packages_dir_path, packages=OrderedDict()
        )

        with caplog.at_level(logging.ERROR), mock.patch(
            "aea.package_manager.v0.check_fingerprint",
            return_value=True,
        ), mock.patch.object(
            pm,
            "iter_dependency_tree",
            return_value=[
                EXAMPLE_PACKAGE_ID,
            ],
        ):
            assert pm.verify() == 1
            assert f"Cannot find hash for {EXAMPLE_PACKAGE_ID}" in caplog.text


@mock.patch("aea.package_manager.base.load_fetch_ipfs")
@mock.patch("aea.package_manager.base.Cache")
def test_package_manager_add_item_dependency_support_mock(*mocks):
    """Check PackageManager.add_packages works with dependencies on mocks."""
    FAKE_PACKAGES = [
        PackageId(
            package_type=PackageType.SKILL,
            public_id=PublicId.from_str(
                f"valory/abstract_{i}:0.1.0:bafybeifh4qtjurq5637ykxexzexca5l4n6t4ujw26tpnern2swajanvhny"
            ),
        )
        for i in range(3)
    ]

    # no deps
    with tempfile.TemporaryDirectory() as tmpdir:
        package_manager = PackageManagerV0(Path(tmpdir))
        with mock.patch.object(package_manager, "calculate_hash_from_package_id"):
            package_manager.add_package(TEST_SKILL_ID)
            assert len(package_manager.packages) == 1

    # deps loading
    with tempfile.TemporaryDirectory() as tmpdir:
        package_manager = PackageManagerV0(Path(tmpdir))
        with mock.patch.object(
            package_manager, "calculate_hash_from_package_id"
        ), mock.patch.object(
            package_manager,
            "get_package_dependencies",
            side_effect=[
                [FAKE_PACKAGES[0]],
                [FAKE_PACKAGES[1]],
                [FAKE_PACKAGES[2]],
                [],
                [],
                [],
                [],
            ],
        ):
            package_manager.add_package(
                TEST_SKILL_ID,
                with_dependencies=True,
            )
            assert len(package_manager.packages) == 4



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tac.proto
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\ttac.proto\x12\x16\x61\x65\x61.fetchai.tac.v1_0_0"\xf9\x1f\n\nTacMessage\x12N\n\tcancelled\x18\x05 \x01(\x0b\x32\x39.aea.fetchai.tac.v1_0_0.TacMessage.Cancelled_PerformativeH\x00\x12N\n\tgame_data\x18\x06 \x01(\x0b\x32\x39.aea.fetchai.tac.v1_0_0.TacMessage.Game_Data_PerformativeH\x00\x12L\n\x08register\x18\x07 \x01(\x0b\x32\x38.aea.fetchai.tac.v1_0_0.TacMessage.Register_PerformativeH\x00\x12N\n\ttac_error\x18\x08 \x01(\x0b\x32\x39.aea.fetchai.tac.v1_0_0.TacMessage.Tac_Error_PerformativeH\x00\x12R\n\x0btransaction\x18\t \x01(\x0b\x32;.aea.fetchai.tac.v1_0_0.TacMessage.Transaction_PerformativeH\x00\x12l\n\x18transaction_confirmation\x18\n \x01(\x0b\x32H.aea.fetchai.tac.v1_0_0.TacMessage.Transaction_Confirmation_PerformativeH\x00\x12P\n\nunregister\x18\x0b \x01(\x0b\x32:.aea.fetchai.tac.v1_0_0.TacMessage.Unregister_PerformativeH\x00\x1a\x89\x03\n\tErrorCode\x12N\n\nerror_code\x18\x01 \x01(\x0e\x32:.aea.fetchai.tac.v1_0_0.TacMessage.ErrorCode.ErrorCodeEnum"\xab\x02\n\rErrorCodeEnum\x12\x11\n\rGENERIC_ERROR\x10\x00\x12\x15\n\x11REQUEST_NOT_VALID\x10\x01\x12!\n\x1d\x41GENT_ADDR_ALREADY_REGISTERED\x10\x02\x12!\n\x1d\x41GENT_NAME_ALREADY_REGISTERED\x10\x03\x12\x18\n\x14\x41GENT_NOT_REGISTERED\x10\x04\x12\x19\n\x15TRANSACTION_NOT_VALID\x10\x05\x12\x1c\n\x18TRANSACTION_NOT_MATCHING\x10\x06\x12\x1f\n\x1b\x41GENT_NAME_NOT_IN_WHITELIST\x10\x07\x12\x1b\n\x17\x43OMPETITION_NOT_RUNNING\x10\x08\x12\x19\n\x15\x44IALOGUE_INCONSISTENT\x10\t\x1a+\n\x15Register_Performative\x12\x12\n\nagent_name\x18\x01 \x01(\t\x1a\x19\n\x17Unregister_Performative\x1a\xc8\x05\n\x18Transaction_Performative\x12\x16\n\x0etransaction_id\x18\x01 \x01(\t\x12\x11\n\tledger_id\x18\x02 \x01(\t\x12\x16\n\x0esender_address\x18\x03 \x01(\t\x12\x1c\n\x14\x63ounterparty_address\x18\x04 \x01(\t\x12r\n\x15\x61mount_by_currency_id\x18\x05 \x03(\x0b\x32S.aea.fetchai.tac.v1_0_0.TacMessage.Transaction_Performative.AmountByCurrencyIdEntry\x12l\n\x12\x66\x65\x65_by_currency_id\x18\x06 \x03(\x0b\x32P.aea.fetchai.tac.v1_0_0.TacMessage.Transaction_Performative.FeeByCurrencyIdEntry\x12r\n\x15quantities_by_good_id\x18\x07 \x03(\x0b\x32S.aea.fetchai.tac.v1_0_0.TacMessage.Transaction_Performative.QuantitiesByGoodIdEntry\x12\r\n\x05nonce\x18\x08 \x01(\t\x12\x18\n\x10sender_signature\x18\t \x01(\t\x12\x1e\n\x16\x63ounterparty_signature\x18\n \x01(\t\x1a\x39\n\x17\x41mountByCurrencyIdEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\x1a\x36\n\x14\x46\x65\x65\x42yCurrencyIdEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\x1a\x39\n\x17QuantitiesByGoodIdEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\x1a\x18\n\x16\x43\x61ncelled_Performative\x1a\xa3\x0c\n\x16Game_Data_Performative\x12p\n\x15\x61mount_by_currency_id\x18\x01 \x03(\x0b\x32Q.aea.fetchai.tac.v1_0_0.TacMessage.Game_Data_Performative.AmountByCurrencyIdEntry\x12\x81\x01\n\x1e\x65xchange_params_by_currency_id\x18\x02 \x03(\x0b\x32Y.aea.fetchai.tac.v1_0_0.TacMessage.Game_Data_Performative.ExchangeParamsByCurrencyIdEntry\x12p\n\x15quantities_by_good_id\x18\x03 \x03(\x0b\x32Q.aea.fetchai.tac.v1_0_0.TacMessage.Game_Data_Performative.QuantitiesByGoodIdEntry\x12w\n\x19utility_params_by_good_id\x18\x04 \x03(\x0b\x32T.aea.fetchai.tac.v1_0_0.TacMessage.Game_Data_Performative.UtilityParamsByGoodIdEntry\x12j\n\x12\x66\x65\x65_by_currency_id\x18\x05 \x03(\x0b\x32N.aea.fetchai.tac.v1_0_0.TacMessage.Game_Data_Performative.FeeByCurrencyIdEntry\x12j\n\x12\x61gent_addr_to_name\x18\x06 \x03(\x0b\x32N.aea.fetchai.tac.v1_0_0.TacMessage.Game_Data_Performative.AgentAddrToNameEntry\x12l\n\x13\x63urrency_id_to_name\x18\x07 \x03(\x0b\x32O.aea.fetchai.tac.v1_0_0.TacMessage.Game_Data_Performative.CurrencyIdToNameEntry\x12\x64\n\x0fgood_id_to_name\x18\x08 \x03(\x0b\x32K.aea.fetchai.tac.v1_0_0.TacMessage.Game_Data_Performative.GoodIdToNameEntry\x12\x12\n\nversion_id\x18\t \x01(\t\x12Q\n\x04info\x18\n \x03(\x0b\x32\x43.aea.fetchai.tac.v1_0_0.TacMessage.Game_Data_Performative.InfoEntry\x12\x13\n\x0binfo_is_set\x18\x0b \x01(\x08\x1a\x39\n\x17\x41mountByCurrencyIdEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\x1a\x41\n\x1f\x45xchangeParamsByCurrencyIdEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x01:\x02\x38\x01\x1a\x39\n\x17QuantitiesByGoodIdEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\x1a<\n\x1aUtilityParamsByGoodIdEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x01:\x02\x38\x01\x1a\x36\n\x14\x46\x65\x65\x42yCurrencyIdEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\x1a\x36\n\x14\x41gentAddrToNameEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x37\n\x15\x43urrencyIdToNameEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x33\n\x11GoodIdToNameEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a+\n\tInfoEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\xb7\x03\n%Transaction_Confirmation_Performative\x12\x16\n\x0etransaction_id\x18\x01 \x01(\t\x12\x7f\n\x15\x61mount_by_currency_id\x18\x02 \x03(\x0b\x32`.aea.fetchai.tac.v1_0_0.TacMessage.Transaction_Confirmation_Performative.AmountByCurrencyIdEntry\x12\x7f\n\x15quantities_by_good_id\x18\x03 \x03(\x0b\x32`.aea.fetchai.tac.v1_0_0.TacMessage.Transaction_Confirmation_Performative.QuantitiesByGoodIdEntry\x1a\x39\n\x17\x41mountByCurrencyIdEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\x1a\x39\n\x17QuantitiesByGoodIdEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\x1a\xef\x01\n\x16Tac_Error_Performative\x12@\n\nerror_code\x18\x01 \x01(\x0b\x32,.aea.fetchai.tac.v1_0_0.TacMessage.ErrorCode\x12Q\n\x04info\x18\x02 \x03(\x0b\x32\x43.aea.fetchai.tac.v1_0_0.TacMessage.Tac_Error_Performative.InfoEntry\x12\x13\n\x0binfo_is_set\x18\x03 \x01(\x08\x1a+\n\tInfoEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x42\x0e\n\x0cperformativeb\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "tac_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _TACMESSAGE_TRANSACTION_PERFORMATIVE_AMOUNTBYCURRENCYIDENTRY._options = None
    _TACMESSAGE_TRANSACTION_PERFORMATIVE_AMOUNTBYCURRENCYIDENTRY._serialized_options = (
        b"8\001"
    )
    _TACMESSAGE_TRANSACTION_PERFORMATIVE_FEEBYCURRENCYIDENTRY._options = None
    _TACMESSAGE_TRANSACTION_PERFORMATIVE_FEEBYCURRENCYIDENTRY._serialized_options = (
        b"8\001"
    )
    _TACMESSAGE_TRANSACTION_PERFORMATIVE_QUANTITIESBYGOODIDENTRY._options = None
    _TACMESSAGE_TRANSACTION_PERFORMATIVE_QUANTITIESBYGOODIDENTRY._serialized_options = (
        b"8\001"
    )
    _TACMESSAGE_GAME_DATA_PERFORMATIVE_AMOUNTBYCURRENCYIDENTRY._options = None
    _TACMESSAGE_GAME_DATA_PERFORMATIVE_AMOUNTBYCURRENCYIDENTRY._serialized_options = (
        b"8\001"
    )
    _TACMESSAGE_GAME_DATA_PERFORMATIVE_EXCHANGEPARAMSBYCURRENCYIDENTRY._options = None
    _TACMESSAGE_GAME_DATA_PERFORMATIVE_EXCHANGEPARAMSBYCURRENCYIDENTRY._serialized_options = (
        b"8\001"
    )
    _TACMESSAGE_GAME_DATA_PERFORMATIVE_QUANTITIESBYGOODIDENTRY._options = None
    _TACMESSAGE_GAME_DATA_PERFORMATIVE_QUANTITIESBYGOODIDENTRY._serialized_options = (
        b"8\001"
    )
    _TACMESSAGE_GAME_DATA_PERFORMATIVE_UTILITYPARAMSBYGOODIDENTRY._options = None
    _TACMESSAGE_GAME_DATA_PERFORMATIVE_UTILITYPARAMSBYGOODIDENTRY._serialized_options = (
        b"8\001"
    )
    _TACMESSAGE_GAME_DATA_PERFORMATIVE_FEEBYCURRENCYIDENTRY._options = None
    _TACMESSAGE_GAME_DATA_PERFORMATIVE_FEEBYCURRENCYIDENTRY._serialized_options = (
        b"8\001"
    )
    _TACMESSAGE_GAME_DATA_PERFORMATIVE_AGENTADDRTONAMEENTRY._options = None
    _TACMESSAGE_GAME_DATA_PERFORMATIVE_AGENTADDRTONAMEENTRY._serialized_options = (
        b"8\001"
    )
    _TACMESSAGE_GAME_DATA_PERFORMATIVE_CURRENCYIDTONAMEENTRY._options = None
    _TACMESSAGE_GAME_DATA_PERFORMATIVE_CURRENCYIDTONAMEENTRY._serialized_options = (
        b"8\001"
    )
    _TACMESSAGE_GAME_DATA_PERFORMATIVE_GOODIDTONAMEENTRY._options = None
    _TACMESSAGE_GAME_DATA_PERFORMATIVE_GOODIDTONAMEENTRY._serialized_options = b"8\001"
    _TACMESSAGE_GAME_DATA_PERFORMATIVE_INFOENTRY._options = None
    _TACMESSAGE_GAME_DATA_PERFORMATIVE_INFOENTRY._serialized_options = b"8\001"
    _TACMESSAGE_TRANSACTION_CONFIRMATION_PERFORMATIVE_AMOUNTBYCURRENCYIDENTRY._options = (
        None
    )
    _TACMESSAGE_TRANSACTION_CONFIRMATION_PERFORMATIVE_AMOUNTBYCURRENCYIDENTRY._serialized_options = (
        b"8\001"
    )
    _TACMESSAGE_TRANSACTION_CONFIRMATION_PERFORMATIVE_QUANTITIESBYGOODIDENTRY._options = (
        None
    )
    _TACMESSAGE_TRANSACTION_CONFIRMATION_PERFORMATIVE_QUANTITIESBYGOODIDENTRY._serialized_options = (
        b"8\001"
    )
    _TACMESSAGE_TAC_ERROR_PERFORMATIVE_INFOENTRY._options = None
    _TACMESSAGE_TAC_ERROR_PERFORMATIVE_INFOENTRY._serialized_options = b"8\001"
    _globals["_TACMESSAGE"]._serialized_start = 38
    _globals["_TACMESSAGE"]._serialized_end = 4127
    _globals["_TACMESSAGE_ERRORCODE"]._serialized_start = 647
    _globals["_TACMESSAGE_ERRORCODE"]._serialized_end = 1040
    _globals["_TACMESSAGE_ERRORCODE_ERRORCODEENUM"]._serialized_start = 741
    _globals["_TACMESSAGE_ERRORCODE_ERRORCODEENUM"]._serialized_end = 1040
    _globals["_TACMESSAGE_REGISTER_PERFORMATIVE"]._serialized_start = 1042
    _globals["_TACMESSAGE_REGISTER_PERFORMATIVE"]._serialized_end = 1085
    _globals["_TACMESSAGE_UNREGISTER_PERFORMATIVE"]._serialized_start = 1087
    _globals["_TACMESSAGE_UNREGISTER_PERFORMATIVE"]._serialized_end = 1112
    _globals["_TACMESSAGE_TRANSACTION_PERFORMATIVE"]._serialized_start = 1115
    _globals["_TACMESSAGE_TRANSACTION_PERFORMATIVE"]._serialized_end = 1827
    _globals[
        "_TACMESSAGE_TRANSACTION_PERFORMATIVE_AMOUNTBYCURRENCYIDENTRY"
    ]._serialized_start = 1655
    _globals[
        "_TACMESSAGE_TRANSACTION_PERFORMATIVE_AMOUNTBYCURRENCYIDENTRY"
    ]._serialized_end = 1712
    _globals[
        "_TACMESSAGE_TRANSACTION_PERFORMATIVE_FEEBYCURRENCYIDENTRY"
    ]._serialized_start = 1714
    _globals[
        "_TACMESSAGE_TRANSACTION_PERFORMATIVE_FEEBYCURRENCYIDENTRY"
    ]._serialized_end = 1768
    _globals[
        "_TACMESSAGE_TRANSACTION_PERFORMATIVE_QUANTITIESBYGOODIDENTRY"
    ]._serialized_start = 1770
    _globals[
        "_TACMESSAGE_TRANSACTION_PERFORMATIVE_QUANTITIESBYGOODIDENTRY"
    ]._serialized_end = 1827
    _globals["_TACMESSAGE_CANCELLED_PERFORMATIVE"]._serialized_start = 1829
    _globals["_TACMESSAGE_CANCELLED_PERFORMATIVE"]._serialized_end = 1853
    _globals["_TACMESSAGE_GAME_DATA_PERFORMATIVE"]._serialized_start = 1856
    _globals["_TACMESSAGE_GAME_DATA_PERFORMATIVE"]._serialized_end = 3427
    _globals[
        "_TACMESSAGE_GAME_DATA_PERFORMATIVE_AMOUNTBYCURRENCYIDENTRY"
    ]._serialized_start = 1655
    _globals[
        "_TACMESSAGE_GAME_DATA_PERFORMATIVE_AMOUNTBYCURRENCYIDENTRY"
    ]._serialized_end = 1712
    _globals[
        "_TACMESSAGE_GAME_DATA_PERFORMATIVE_EXCHANGEPARAMSBYCURRENCYIDENTRY"
    ]._serialized_start = 2974
    _globals[
        "_TACMESSAGE_GAME_DATA_PERFORMATIVE_EXCHANGEPARAMSBYCURRENCYIDENTRY"
    ]._serialized_end = 3039
    _globals[
        "_TACMESSAGE_GAME_DATA_PERFORMATIVE_QUANTITIESBYGOODIDENTRY"
    ]._serialized_start = 1770
    _globals[
        "_TACMESSAGE_GAME_DATA_PERFORMATIVE_QUANTITIESBYGOODIDENTRY"
    ]._serialized_end = 1827
    _globals[
        "_TACMESSAGE_GAME_DATA_PERFORMATIVE_UTILITYPARAMSBYGOODIDENTRY"
    ]._serialized_start = 3100
    _globals[
        "_TACMESSAGE_GAME_DATA_PERFORMATIVE_UTILITYPARAMSBYGOODIDENTRY"
    ]._serialized_end = 3160
    _globals[
        "_TACMESSAGE_GAME_DATA_PERFORMATIVE_FEEBYCURRENCYIDENTRY"
    ]._serialized_start = 1714
    _globals[
        "_TACMESSAGE_GAME_DATA_PERFORMATIVE_FEEBYCURRENCYIDENTRY"
    ]._serialized_end = 1768
    _globals[
        "_TACMESSAGE_GAME_DATA_PERFORMATIVE_AGENTADDRTONAMEENTRY"
    ]._serialized_start = 3218
    _globals[
        "_TACMESSAGE_GAME_DATA_PERFORMATIVE_AGENTADDRTONAMEENTRY"
    ]._serialized_end = 3272
    _globals[
        "_TACMESSAGE_GAME_DATA_PERFORMATIVE_CURRENCYIDTONAMEENTRY"
    ]._serialized_start = 3274
    _globals[
        "_TACMESSAGE_GAME_DATA_PERFORMATIVE_CURRENCYIDTONAMEENTRY"
    ]._serialized_end = 3329
    _globals[
        "_TACMESSAGE_GAME_DATA_PERFORMATIVE_GOODIDTONAMEENTRY"
    ]._serialized_start = 3331
    _globals["_TACMESSAGE_GAME_DATA_PERFORMATIVE_GOODIDTONAMEENTRY"]._serialized_end = (
        3382
    )
    _globals["_TACMESSAGE_GAME_DATA_PERFORMATIVE_INFOENTRY"]._serialized_start = 3384
    _globals["_TACMESSAGE_GAME_DATA_PERFORMATIVE_INFOENTRY"]._serialized_end = 3427
    _globals["_TACMESSAGE_TRANSACTION_CONFIRMATION_PERFORMATIVE"]._serialized_start = (
        3430
    )
    _globals["_TACMESSAGE_TRANSACTION_CONFIRMATION_PERFORMATIVE"]._serialized_end = 3869
    _globals[
        "_TACMESSAGE_TRANSACTION_CONFIRMATION_PERFORMATIVE_AMOUNTBYCURRENCYIDENTRY"
    ]._serialized_start = 1655
    _globals[
        "_TACMESSAGE_TRANSACTION_CONFIRMATION_PERFORMATIVE_AMOUNTBYCURRENCYIDENTRY"
    ]._serialized_end = 1712
    _globals[
        "_TACMESSAGE_TRANSACTION_CONFIRMATION_PERFORMATIVE_QUANTITIESBYGOODIDENTRY"
    ]._serialized_start = 1770
    _globals[
        "_TACMESSAGE_TRANSACTION_CONFIRMATION_PERFORMATIVE_QUANTITIESBYGOODIDENTRY"
    ]._serialized_end = 1827
    _globals["_TACMESSAGE_TAC_ERROR_PERFORMATIVE"]._serialized_start = 3872
    _globals["_TACMESSAGE_TAC_ERROR_PERFORMATIVE"]._serialized_end = 4111
    _globals["_TACMESSAGE_TAC_ERROR_PERFORMATIVE_INFOENTRY"]._serialized_start = 3384
    _globals["_TACMESSAGE_TAC_ERROR_PERFORMATIVE_INFOENTRY"]._serialized_end = 3427
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2025-2026 Valory AG
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""This package contains handlers for the fipa dummy buyer skill."""

from typing import Optional, cast

from aea.configurations.base import PublicId
from aea.protocols.base import Message
from aea.skills.base import Handler

from packages.fetchai.protocols.fipa.message import FipaMessage
from packages.fetchai.skills.fipa_dummy_buyer.dialogues import FipaDialogues


class FipaBuyerHandler(Handler):
    """This class implements a FIPA handler."""

    SUPPORTED_PROTOCOL: Optional[PublicId] = FipaMessage.protocol_id

    def setup(self) -> None:
        """Implement the setup."""

    def handle(self, message: Message) -> None:
        """Handle an evelope."""
        dialogues = cast(FipaDialogues, self.context.fipa_dialogues)
        if message.performative == FipaMessage.Performative.PROPOSE:
            buyer_dialogue = dialogues.update(message)
            if not buyer_dialogue:
                self.context.logger.error("error on propose message dialogue update")
                return
            # got a message, switch off initializer
            self.context.behaviours.initializer.is_enabled = False
            accept_msg = buyer_dialogue.reply(
                performative=FipaMessage.Performative.ACCEPT, target_message=message
            )
            self.context.outbox.put_message(message=accept_msg)
        elif message.performative == FipaMessage.Performative.MATCH_ACCEPT:
            buyer_dialogue = dialogues.update(message)
            if not buyer_dialogue:
                self.context.logger.error(
                    "error on MATCH_ACCEPT message dialogue update"
                )
                return
            end_msg = buyer_dialogue.reply(
                performative=FipaMessage.Performative.END, target_message=message
            )
            self.context.outbox.put_message(message=end_msg)
            self.context.logger.info("FIPA INTERACTION COMPLETE")
        else:
            self.context.logger.error(
                f"unsupported performative: {message.performative}"
            )

    def teardown(self) -> None:
        """Implement the handler teardown."""



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: crypto.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0c\x63rypto.proto\x12\tcrypto.pb\"?\n\tPublicKey\x12$\n\x08key_type\x18\x01 \x02(\x0e\x32\x12.crypto.pb.KeyType\x12\x0c\n\x04\x64\x61ta\x18\x02 \x02(\x0c\"@\n\nPrivateKey\x12$\n\x08key_type\x18\x01 \x02(\x0e\x32\x12.crypto.pb.KeyType\x12\x0c\n\x04\x64\x61ta\x18\x02 \x02(\x0c*9\n\x07KeyType\x12\x07\n\x03RSA\x10\x00\x12\x0b\n\x07\x45\x64\x32\x35\x35\x31\x39\x10\x01\x12\r\n\tSecp256k1\x10\x02\x12\t\n\x05\x45\x43\x44SA\x10\x03')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'crypto_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_KEYTYPE']._serialized_start=158
  _globals['_KEYTYPE']._serialized_end=215
  _globals['_PUBLICKEY']._serialized_start=27
  _globals['_PUBLICKEY']._serialized_end=90
  _globals['_PRIVATEKEY']._serialized_start=92
  _globals['_PRIVATEKEY']._serialized_end=156
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Test helper utils."""

import socket
from unittest import mock

from aea.test_tools.network import (
    LOCALHOST,
    get_host,
    get_unused_tcp_port,
    is_port_in_use,
)


def test_get_unused_tcp_port() -> None:
    """Test get_unused_tcp_port"""

    n_ports = 100
    for _ in range(n_ports):
        assert not is_port_in_use(get_unused_tcp_port())


def test_get_host():
    """Test get_host"""

    assert socket.inet_aton(get_host())
    with mock.patch("socket.socket.connect", side_effect=Exception):
        assert socket.inet_aton(get_host())
        assert get_host() == LOCALHOST.hostname



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022-2026 Valory AG
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------


"""This module contains the tests for the aea.configurations.loader module."""

import os
from collections import OrderedDict
from io import StringIO
from pathlib import Path
from tempfile import TemporaryDirectory
from unittest import mock
from unittest.mock import MagicMock

import pytest
import yaml

import aea
import aea.configurations
from aea.configurations.base import (
    AgentConfig,
    PackageConfiguration,
    PackageType,
    ProtocolSpecification,
)
from aea.configurations.loader import (
    ConfigLoader,
    load_protocol_specification_from_string,
)
from aea.configurations.validation import make_jsonschema_base_uri
from aea.exceptions import AEAEnforceError
from aea.protocols.generator.common import load_protocol_specification

from tests.conftest import ROOT_DIR, protocol_specification_files


def test_windows_uri_path():
    """Test uri path on running platform."""
    path = Path("aea", "configurations").absolute()
    output = make_jsonschema_base_uri(path)

    if os.name == "nt":
        assert output == f"file:///{'/'.join(path.parts)}/"
    else:
        assert output == f"file:/{'/'.join(path.parts)}/"


@pytest.mark.parametrize("aea_config_yaml", Path(ROOT_DIR).rglob("**/aea-config.yaml"))
def test_load_aea_config(aea_config_yaml):
    """Test loading aea_config.yaml file"""
    config_loader = ConfigLoader("aea-config_schema.json", AgentConfig)
    config_loader.load(open(aea_config_yaml))


def test_config_loader_get_required_fields():
    """Test required fields of ConfigLoader."""
    config_loader = ConfigLoader.from_configuration_type(PackageType.PROTOCOL)
    assert config_loader.required_fields


@mock.patch.object(aea.configurations.loader, "yaml_dump")
@mock.patch.object(ConfigLoader, "validate")
@mock.patch("builtins.open")
def test_config_loader_dump_component(*_mocks):
    """Test ConfigLoader.dump"""
    config_loader = ConfigLoader.from_configuration_type(PackageType.PROTOCOL)
    configuration = MagicMock()
    config_loader.dump(configuration, open("foo"))


@mock.patch.object(aea.configurations.loader, "yaml_dump_all")
@mock.patch.object(ConfigLoader, "validate")
@mock.patch("builtins.open")
def test_config_loader_dump_agent_config(*_mocks):
    """Test ConfigLoader.dump"""
    config_loader = ConfigLoader.from_configuration_type(PackageType.AGENT)
    configuration = MagicMock(ordered_json={"component_configurations": []})
    config_loader.dump(configuration, open("foo"))


def test_config_loader_load_service_config(*_mocks):
    """Test ConfigLoader.dump"""

    config = OrderedDict(
        {
            "name": "Service",
            "author": "valory",
            "version": "0.1.0",
            "description": "Description",
            "aea_version": ">=2.0.0, <3.0.0",
            "license": "Apache-2.0",
            "agent": "agent",
            "network": "hardhat",
            "number_of_agents": "4",
            "overrides": [],
        }
    )

    class DummyServiceConfig(PackageConfiguration):
        package_type = PackageType.SERVICE
        json = property(lambda self: config)

        def pop(self, item):
            return self.json.pop(item)

    dummy_service_config = DummyServiceConfig(name="service", author="valory")

    with TemporaryDirectory() as temp_dir:
        schema_file = Path(temp_dir, "schema.json").absolute()
        config_file = Path(temp_dir, "service.yaml")
        schema_file.write_text("{}")

        config_loader_cls = DummyServiceConfig
        config_loader_cls.schema = str(schema_file)
        config_loader_cls.from_json = lambda x: dummy_service_config
        config_loader = ConfigLoader.from_configuration_type(
            PackageType.SERVICE, {PackageType.SERVICE: config_loader_cls}
        )

        config_file.touch()
        with pytest.raises(ValueError, match="Service configuration file was empty."):
            config_loader.load(config_file.open("r"))

        config_loader.dump(dummy_service_config, config_file.open("w+"))
        service_config = config_loader.load(config_file.open("r"))
        assert any(getattr(service_config, key) == val for key, val in config.items())


@pytest.mark.parametrize("spec_file_path", protocol_specification_files)
def test_load_protocol_specification(spec_file_path):
    """Test for the utility function 'load_protocol_specification'"""
    result = load_protocol_specification(spec_file_path)
    assert isinstance(result, ProtocolSpecification)


@pytest.mark.parametrize("spec_file_path", protocol_specification_files)
def test_load_protocol_specification_from_string(spec_file_path):
    """Test for the utility function 'load_protocol_specification_from_string'"""
    with open(spec_file_path, "r") as f:
        load_protocol_specification_from_string(f.read())


@mock.patch("aea.protocols.generator.common.open_file")
@mock.patch.object(ConfigLoader, "validate")
def test_load_protocol_specification_only_first_part(*_mocks):
    """Test 'load_protocol_specification' with only the first part."""
    valid_protocol_specification = dict(
        name="name",
        author="author",
        version="0.1.0",
        license="",
        aea_version="0.1.0",
        speech_acts={"example": {}},
        protocol_specification_id="test/test:0.1.0",
        description="some",
    )
    with mock.patch.object(
        yaml, "safe_load_all", return_value=[valid_protocol_specification]
    ):
        load_protocol_specification("foo")


@mock.patch("aea.protocols.generator.common.open_file")
@mock.patch.object(ConfigLoader, "validate")
def test_load_protocol_specification_two_parts(*_mocks):
    """Test 'load_protocol_specification' with two parts."""
    valid_protocol_specification = dict(
        name="name",
        author="author",
        version="0.1.0",
        license="",
        aea_version="0.1.0",
        speech_acts={"example": {}},
        protocol_specification_id="test/test:0.1.0",
        description="some",
    )
    with mock.patch.object(
        yaml,
        "safe_load_all",
        return_value=[valid_protocol_specification, valid_protocol_specification],
    ):
        load_protocol_specification("foo")


def test_load_protocol_specification_too_many_parts():
    """Test 'load_protocol_specification' with more than three parts."""
    with pytest.raises(
        ValueError,
        match="Incorrect number of Yaml documents in the protocol specification.",
    ):
        with mock.patch.object(
            yaml, "safe_load_all", return_value=[{}] * 4
        ), mock.patch("aea.protocols.generator.common.open_file"):
            load_protocol_specification("foo")


@mock.patch.object(aea, "__version__", "0.1.0")
def test_load_package_configuration_with_incompatible_aea_version(*_mocks):
    """Test that loading a package configuration with incompatible AEA version raises an error."""
    config_loader = ConfigLoader.from_configuration_type(
        PackageType.PROTOCOL, skip_aea_validation=False
    )
    specifier_set = "<3.0.0,>=2.0.0"
    file = StringIO(f"name: some_protocol\naea_version: '{specifier_set}'")
    with pytest.raises(
        AEAEnforceError,
        match=f"AEA version in use '0.1.0' is not compatible with the specifier set '{specifier_set}'.",
    ):
        config_loader.load(file)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022-2026 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Test package manager base."""

import logging
import re
from collections import OrderedDict
from pathlib import Path
from typing import Callable, Dict, Optional
from typing import OrderedDict as OrderedDictType
from unittest import mock

import pytest

from aea.configurations.base import PackageConfiguration
from aea.configurations.constants import PACKAGES
from aea.configurations.data_types import ComponentId, PackageId, PackageType, PublicId
from aea.package_manager.base import (
    BasePackageManager,
    ConfigLoaderCallableType,
    DepedencyMismatchErrors,
    PackageUpdateError,
    load_configuration,
)
from aea.test_tools.test_cases import BaseAEATestCase

from tests.conftest import ROOT_DIR

EXAMPLE_PACKAGE_ID = PackageId(
    package_type=PackageType.PROTOCOL,
    public_id=PublicId(author="open_aea", name="signing", version="1.0.0"),
)
DUMMY_PACKAGE_ID = PackageId(
    package_type=PackageType.SKILL,
    public_id=PublicId(author="dummy", name="name"),
)
EXAMPLE_PACKAGE_HASH = "bafybeihgwrowgtegpe4ixe3iy5jx65u7pxebl5btrkywzmx7g52gjtbnpi"
DUMMY_PACKAGE_HASH = "bafybei0000000000000000000000000000000000000000000000000000"
PACKAGE_JSON_FILE = Path(ROOT_DIR, PACKAGES, "packages.json")


def _dummy_loader(
    package_type: PackageType, package_path: Path
) -> PackageConfiguration:
    """Dummy loader"""

    class _DummyPackageConfig(PackageConfiguration):
        """Dummy class."""

        @property
        def json(self) -> Dict:
            """JSON"""

            return {}

    return _DummyPackageConfig(name="name", author="author")


def _fetch_ipfs_mock(_, __, dest: str, ___) -> None:
    """fetch_ipfs method mock."""
    Path(dest).mkdir()


class DummyPackageManager(BasePackageManager):
    """Dummy package manager."""

    def __init__(
        self,
        path: Path,
        packages: Dict[PackageId, str],
        config_loader: ConfigLoaderCallableType = load_configuration,
    ) -> None:
        """Package hashes."""

        self.path = path
        self.packages = packages
        self.config_loader = config_loader
        self.cache = mock.Mock(exists=lambda *x: False)
        self._logger = logging.getLogger()

    @classmethod
    def from_dir(
        cls,
        packages_dir: Path,
        config_loader: ConfigLoaderCallableType = load_configuration,
    ) -> "BasePackageManager":
        """Load from dir."""

        return cls(path=packages_dir, packages={}, config_loader=config_loader)

    def get_package_hash(self, package_id: PackageId) -> Optional[str]:
        """Return package hash."""

        return self.packages.get(package_id)

    @property
    def json(self) -> OrderedDictType:
        """Return json rep"""
        return OrderedDict(self.packages)

    def sync(  # type: ignore[empty-body]
        self,
        dev: bool = False,
        third_party: bool = True,
        update_packages: bool = False,
        update_hashes: bool = False,
    ) -> "BasePackageManager":
        """Perorm sync."""

    def update_package_hashes(  # type: ignore[empty-body]
        self,
        selector_prompt: Optional[Callable[[], str]] = None,
        skip_missing: bool = False,
    ) -> "BasePackageManager":
        """Update package hashes."""

    def verify(  # type: ignore[empty-body]
        self,
    ) -> int:
        """Verify hashes."""

    def register(
        self, package_path: Path, package_type: Optional[PackageType] = None
    ) -> "DummyPackageManager":
        """Register package"""
        return self


class TestBaseManager(BaseAEATestCase):
    """Test base implementation."""

    use_packages_dir: bool = True

    def test_initialization(
        self,
    ) -> None:
        """Test manager initialization."""
        packages = {EXAMPLE_PACKAGE_ID: EXAMPLE_PACKAGE_HASH}
        package_manager = DummyPackageManager(
            path=self.packages_dir_path,
            packages=packages,
        )

        assert len(package_manager.packages) == len(packages)
        assert package_manager.path == self.packages_dir_path

    def test_package_id_to_path(
        self,
    ) -> None:
        """Test package id to path method."""

        package_manager = DummyPackageManager(
            path=self.packages_dir_path,
            packages={},
        )

        assert package_manager.package_path_from_package_id(EXAMPLE_PACKAGE_ID) == Path(
            self.packages_dir_path,
            EXAMPLE_PACKAGE_ID.author,
            EXAMPLE_PACKAGE_ID.package_type.to_plural(),
            EXAMPLE_PACKAGE_ID.name,
        )

    def test_add_package(
        self,
    ) -> None:
        """Test add package method."""

        package_manager = DummyPackageManager(
            path=self.packages_dir_path,
            packages={},
        )
        dummy_package = PackageId(
            package_type=PackageType.SKILL,
            public_id=PublicId(
                author="dummy",
                name="skill",
                package_hash="bafybeicoawiackcbgqo3na3e56tpdc62atag4yxknur77py37caqq4mmya",
            ),
        )
        with mock.patch(
            "aea.package_manager.base.load_fetch_ipfs", new=lambda: _fetch_ipfs_mock
        ):
            package_manager.add_package(
                package_id=dummy_package,
            )

            assert (self.packages_dir_path / dummy_package.author).exists()
            assert (
                self.packages_dir_path
                / dummy_package.author
                / dummy_package.package_type.to_plural()
            ).exists()
            assert (
                self.packages_dir_path
                / dummy_package.author
                / dummy_package.package_type.to_plural()
                / dummy_package.name
            ).exists()

    def test_update_package_failure(
        self,
    ) -> None:
        """Test update package method."""

        package_manager = DummyPackageManager(
            path=self.packages_dir_path,
            packages={},
        )

        with mock.patch("shutil.rmtree", side_effect=OSError):
            with pytest.raises(
                PackageUpdateError,
                match=re.escape("Cannot update (protocol, open_aea/signing:1.0.0)"),
            ):
                package_manager.update_package(package_id=EXAMPLE_PACKAGE_ID)

    @pytest.mark.parametrize(
        argnames=("packages", "error_check"),
        argvalues=[
            ({}, DepedencyMismatchErrors.HASH_NOT_FOUND),
            (
                {EXAMPLE_PACKAGE_ID: "fake_hash"},
                DepedencyMismatchErrors.HASH_DOES_NOT_MATCH,
            ),
        ],
    )
    def test_check_dependencies_errors(
        self, packages: Dict, error_check: DepedencyMismatchErrors
    ) -> None:
        """Test check dependencies method."""

        package_manager = DummyPackageManager(
            path=self.packages_dir_path,
            packages=packages,
        )

        failed_checks = package_manager.check_dependencies(
            configuration=mock.MagicMock(
                package_dependencies=[
                    ComponentId(
                        component_type=str(PackageType.PROTOCOL),
                        public_id=PublicId(
                            author="open_aea",
                            name="signing",
                            version="1.0.0",
                            package_hash=EXAMPLE_PACKAGE_HASH,
                        ),
                    )
                ]
            )
        )

        assert len(failed_checks) == 1, failed_checks
        ((package_id, error_found),) = failed_checks

        assert package_id == EXAMPLE_PACKAGE_ID
        assert error_check == error_found

    def test_update_public_id_hash(
        self,
    ) -> None:
        """Test update public id hash method."""

        package_manager = DummyPackageManager(
            path=self.packages_dir_path,
            packages={EXAMPLE_PACKAGE_ID: DUMMY_PACKAGE_HASH},
        )

        updated_public_id = package_manager.update_public_id_hash(
            public_id_str=str(EXAMPLE_PACKAGE_ID.public_id),
            package_type=PackageType.PROTOCOL.value,
        )

        assert PublicId.from_str(updated_public_id).hash == DUMMY_PACKAGE_HASH

    def test_update_dependencies(
        self,
    ) -> None:
        """Test update public id hash method."""

        package_to_update = PackageId(
            package_type=PackageType.SKILL,
            public_id=PublicId.from_str("valory/some_skill"),
        )

        package_manager = DummyPackageManager(
            path=self.packages_dir_path,
            packages={
                EXAMPLE_PACKAGE_ID: DUMMY_PACKAGE_HASH,
                package_to_update: EXAMPLE_PACKAGE_HASH,
            },
        )

        with mock.patch(
            "aea.package_manager.base.load_yaml",
            return_value=[
                {
                    "skills": [
                        "valory/some_skill",
                    ]
                },
                None,
            ],
        ), mock.patch(
            "aea.package_manager.base.dump_yaml",
        ) as dump_patch:
            package_manager.update_dependencies(
                package_id=package_to_update,
            )

            dump_patch.assert_called_once_with(
                file_path=(
                    self.packages_dir_path
                    / package_to_update.author
                    / package_to_update.package_type.to_plural()
                    / package_to_update.name
                    / "skill.yaml"
                ),
                data={
                    "skills": [
                        str(package_to_update.public_id.with_hash(EXAMPLE_PACKAGE_HASH))
                    ]
                },
                extra_data=None,
            )



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""This module contains tests for mocking utils."""

import subprocess  # nosec
from typing import Any
from unittest import mock

import pytest

from aea.test_tools.mocks import AnyStringWith, RegexComparator, ctx_mock_Popen


@pytest.mark.parametrize("builtin_type", [str, list, tuple])
def test_any_string_with(builtin_type: Any) -> None:
    """Test AnyStringWith"""

    assert AnyStringWith("in") in builtin_type(["string"])
    assert not AnyStringWith("ni") in builtin_type(["string"])


@pytest.mark.parametrize("builtin_type", [str, list, tuple])
def test_regex_comparator(builtin_type: Any) -> None:
    """Test RegexComparator"""

    assert RegexComparator("in") in builtin_type(["string"])
    assert not RegexComparator("ni") in builtin_type(["string"])


def test_ctx_mock_popen_communicate_return_value() -> None:
    """Test ctx_mock_popen"""

    cmd_name, *args = "python", "--version"
    popen = subprocess.Popen([cmd_name, *args])  # nosec
    stdout, stderr = popen.communicate()
    assert stdout is stderr is None
    assert popen.returncode == 0

    with ctx_mock_Popen():
        popen = subprocess.Popen([cmd_name])  # nosec
        stdout, stderr = popen.communicate()
        objects = stdout, stderr, popen.returncode
        assert all(isinstance(obj, mock.MagicMock) for obj in objects)
